/***************************************************************************/
/*                                                                         */
/* File:   draw.cpp                                                        */
/* Author: bkenwright@xbdev.net                                            */
/* Date:   10-11-2002                                                      */
/*                                                                         */
/***************************************************************************/



#pragma comment(lib, "kernel32.lib")
#pragma comment(lib, "user32.lib")
#pragma comment(lib, "gdi32.lib")
#pragma comment(lib, "comctl32.lib")
#pragma comment(lib, "Advapi32.lib")
#pragma comment(lib, "Winmm.lib")

#include <windows.h>
#include <stdio.h>

#define ULONG_PTR unsigned long

#pragma comment(lib, "D3d8.lib")
#pragma comment(lib, "D3dx8.lib")
#include <d3dx8.h>

#include "dxdbg.h"

#include "md2.h"



#define SZ_MD2_FILE		"media\\pac3D.md2"
#define SZ_MD2_TEXTURE	"media\\pac3D.bmp"

CMD2X g_md2; // Important piece of code here! ->Using our CLoadMD2 Class<--


LPDIRECT3D8 g_pD3D = NULL;
LPDIRECT3DDEVICE8 g_pD3DDevice = NULL;

void Create(HWND hWnd)
{
    //First of all, create the main D3D object. If it is created successfully we 
    //should get a pointer to an IDirect3D8 interface.
    g_pD3D = Direct3DCreate8(D3D_SDK_VERSION);
    //Get the current display mode
    D3DDISPLAYMODE d3ddm;
    g_pD3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3ddm);
    //Create a structure to hold the settings for our device
    D3DPRESENT_PARAMETERS d3dpp; 
    ZeroMemory(&d3dpp, sizeof(d3dpp));
    //Fill the structure. 
    //We want our program to be windowed, and set the back buffer to a format
    //that matches our current display mode
    d3dpp.Windowed = TRUE;
    d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
    d3dpp.BackBufferFormat = d3ddm.Format;
    //For depth buffering (e.g.) the z-buffer
    d3dpp.BackBufferCount=1;
    d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
    d3dpp.EnableAutoDepthStencil = TRUE;
    //Create a Direct3D device
    g_pD3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING, &d3dpp, &g_pD3DDevice);
    //Turn on z-buffering
    g_pD3DDevice->SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);

	//Turn on back face culling. This is becuase we want to hide the back of our polygons
    //g_pD3DDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE); //D3DCULL_CCW);
	g_pD3DDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CW);

	//Load our data into our CLoadMD2 class instance.
	g_md2.ImportMD2(SZ_MD2_FILE, SZ_MD2_TEXTURE);   
}

void Release()
{
	g_md2.Release();

	g_pD3DDevice->Release();
    g_pD3DDevice = NULL;
    g_pD3D->Release();
    g_pD3D = NULL;
}
void set_camera()
{
    // [1] D3DTS_VIEW
    D3DXMATRIX v;
    g_pD3DDevice->SetTransform(D3DTS_VIEW, D3DXMatrixLookAtLH(&v, &D3DXVECTOR3(0,0,-90), 
                                                                  &D3DXVECTOR3(0,0,0), 
                                                                  &D3DXVECTOR3(0,1,0)));  
    // [2] D3DTS_PROJECTION
    D3DXMATRIX p;
    g_pD3DDevice->SetTransform( D3DTS_PROJECTION, D3DXMatrixPerspectiveFovLH( &p,  D3DX_PI/4,  1.0f,
                                                                                                 1.0f,200.0f));
};


void Render()
{
	if(!g_pD3DDevice)return;

	set_camera();

	static float iCount = 0.0001f;
	iCount += 0.0005f;
	//int iCount = 0;
	
	
	static float angle = 0.0f;
    angle += 0.001f;
	//~~~*~~~~ Create a matrix
	D3DXMATRIX mx;
    //~~~*~~~~ Do somthing to our empty matrix.
    D3DXMatrixRotationY(&mx, angle ); // angle in radians...eg. 1 degree = PI/180
    //~~~*~~~~ Use the matrix! No use having a matrix if we don't use it!
    g_pD3DDevice->SetTransform(D3DTS_WORLD, &mx);
	

	// Which Animation, e.g. 0 for stand
	//            |          1 for run
	//            |          2 for jump etc. etc.
	//           \|/
    //            |
	g_md2.Animate((UINT)iCount);   // This simply sets up our vertex buffer data in our class instance.

	// Clear the back buffer to a blue color
	g_pD3DDevice->Clear( 0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB(0,0,255), 1.0f, 0 );
	g_pD3DDevice->BeginScene();
	

	g_md2.RenderFrame();    // Renders our model to the screen.


	TriDisplay(g_pD3DDevice, g_md2.GetNumTris());
	FPSDisplay(g_pD3DDevice);
	g_pD3DDevice->EndScene();

	// After rendering the scene we display it.
	g_pD3DDevice->Present( NULL, NULL, NULL, NULL );
	
	
}




